// genesis
//
// demo.g
//
// This top-level script illustrates a population-based genetic
// search technique for estimating the current channel conductances
// for a simple one-compartment cell with 4 conductances.
// 

// user-settable parameters
int n_nodes = 4		        // number of nodes to use
int individuals = 1000          // total number of individuals to try out
int population = 32	        // number of individuals alive at any one time
float stopping_criterion = 20.0 // if fitness exceeds this, we will stop
float crossover_prob = 0.5	// probability of crossover (if crossover
				//	is done, it is done on a parameter
				//	basis weighted by the fitness of the
				//	individuals)
float bit_mutation_prob	= 0.02  // probability of mutating a bit when
				// generating a new individual
randseed 67351			// initialize the random number generator

// genetic search management
include population.g       // functions for GA population-based parameter
			   //   searches

// model-specific files
include siminit.g          // defines parameters of simulation
include fI.g	           // sets up table of currents
include channels.g         // defines the channels
include simcell.g          // functions to load in the cell model
include eval.g		   // functions to evaluate the model

paron -farm -silent 0 -nodes {n_nodes} -output o.out
			   // start up parallel library
echo I am node {mytotalnode}
			   // all nodes print their node number

barrierall		   // first wait for every node to start up

if ({mytotalnode} == 0)    // the master (on node 0) runs the search
    init_master
    echo Starting search at {getdate}
    echo "n_nodes = " {n_nodes}
    echo "# of workers = " {n_nodes - 1}
    echo "individuals = " {individuals}
    echo "population = " {population}
    echo
    pb_search {individuals} {population}
else			   // the workers just do some initialization
    init_worker
end

barrierall 7 1000000	   // workers will sit here waiting for task
			   // assignments from the master
paroff
quit
