// genesis

// population.g:  a basic population-based genetic search 

// variables for the parameter search
float crossover_prob = 0.5	// probability of crossover (if crossover
				//	is done, it is done on a parameter
				//	basis weighted by the fitness of the
				//	individuals)
float bit_mutation_prob	= 0.02  // probability of mutating a bit when
				// generating a new individual
int parameters = 0		// number of parameters (user should set this
				//   before calling pb_search)
int actual_population = 0	// number of individuals that are "alive"
int least_fit			// index of least-fit individual in current
				//   population
float min_fitness = -1e+10      // fitness of the least-fit individual
int most_fit			// index of the most-fit individual in current
				//   population
float max_fitness = -1e+10	// fitness of the most-fit individual

int print_count = 0		// counts how many individuals' fitness values
				//    we have printed out

// subordinate files
include farm.g		        // support for task farming

// declarations of externally defined functions
function evaluate end

// initializes the search
function init_search(master)
	int master

	create neutral /ps
	disable /ps

	create neutral /ps/params
	addfield /ps/params label
	addfield /ps/params scaling
	addfield /ps/params min_value
	addfield /ps/params max_value
	addfield /ps/params bits
	addfield /ps/params value
	createmap /ps/params / {parameters} 1

	if (master)
	    create neutral /ps/gene
	    addfield /ps/gene bits

	    create neutral /ps/pending
	    createmap /ps/gene /ps/pending {parameters} 1
	    createmap /ps/pending / {n_nodes} 1

	    create neutral /ps/population
	    addfield /ps/population fitness
	    createmap /ps/gene /ps/population {parameters} 1
	    createmap /ps/population / {population} 1
	    actual_population = 0

	    init_farm
	end
end

function compute_parameter_values
	int i
	float v, mnv, mxv

	for (i = 0; i < parameters; i = i + 1)
	    v = {getfield /params[{i}] bits} / 65536.0;
	    if ({getfield /params[{i}] scaling} == "log")
		if ({getfield /params[{i}] min_value} <= 0.0)
		    mnv = -10.0;
		else
		    mnv = {log {getfield /params[{i}] min_value}}
		end
		if ({getfield /params[{i}] max_value} <= 0.0)
		    mxv = -10.0;
		else
		    mxv = {log {getfield /params[{i}] max_value}}
		end
	 	v = {exp {v * (mxv - mnv) + mnv}};
	    else
		mnv = {getfield /params[{i}] min_value}
		mxv = {getfield /params[{i}] max_value}
	 	v = {v * (mxv - mnv) + mnv};
	    end
	    setfield /params[{i}] value {v}
	end
end

// recompute_fitness_extremes recomputes which individuals in the population
// are the most and least fit
function recompute_fitness_extremes
	int i

	min_fitness = 1e+10
	max_fitness = -1e+10
	for (i = 0; i < actual_population; i = i + 1)
	    if ({getfield /population[{i}] fitness} < min_fitness)
		least_fit = i
		min_fitness = {getfield /population[{i}] fitness}
	    end
	    if ({getfield /population[{i}] fitness} > max_fitness)
		most_fit = i
		max_fitness = {getfield /population[{i}] fitness}
	    end
	end
end

// worker_task performs the task that the master has doled out to this node
function worker_task (index)
	int index
	float fit

	compute_parameter_values

	// determine that fitness value for this individual
	fit = {evaluate}

	// return result to the master (node 0 in zone 0)
	return_result@0.0 {mytotalnode} {index} {fit}
end


function return_result (node, index, fit)
	int node, index
	int p
	float fit
	int added = 0

	if (actual_population < population)
	    // if we are underpopulated, each new individual stays "alive"
	    least_fit = actual_population
	    min_fitness = -1e+10
	    actual_population = actual_population + 1
	end
	if (fit > min_fitness)
	    // replace the least fit individual with the current individual
	    setfield /population[{least_fit}] fitness {fit}
	    for (p = 0; p < parameters; p = p + 1)
		setfield /population[{least_fit}]/gene[{p}] bits \
		    {getfield /pending[{node}]/gene[{p}] bits}
	    end
	    recompute_fitness_extremes
	    added = 1
	end

	// that node is now free, and can be assigned a new task
	setfield /free[{node}] value 1

	// print out the fitness value of the individual we just evaluated

	// compute the real-valued parameters from the bit-strings (the
	//    "genes")
	compute_parameter_values

	echo "(" -n
	for (p = 0; p < parameters; p = p + 1)
	    if (p != 0)
		echo ", "
	    end
	    echo "    " {getfield /params[{p}] label} " = " \
		{getfield /params[{p}] value} -n
	end
	echo "): "
	
        floatformat %12.2f
        echo {fit} -n
	if (added)
	    echo "*" -n
	else
	    echo " " -n
	end
        floatformat %g
//        if({print_count} % 6 == 5)  // 6 fitness values per column
//          echo
//        end
	echo
	print_count = {print_count} + 1
end


// mutate alters the given bit-string based on the mutation probability,
//	and returns the possibly-mutated string
function mutate (v)
    int v
    int i, b

    b = 1
    for (i = 0; i < 16; i = i + 1)
	if ({rand 0 1} < bit_mutation_prob)
	    v = v ^ b
	end
	b = b + b
    end
    return {v}
end

// this prints out the final search results
function print_results
    int i

    echo
    echo "Finished search at " {getdate}

    for (i = 0; i < parameters; i = i + 1)
      setfield /params[{i}] bits \
	{getfield /population[{most_fit}]/gene[{i}] bits}
    end

    compute_parameter_values

    echo "Best match was with"
    for (i = 0; i < parameters; i = i + 1)
      echo "    " {getfield /params[{i}] label} " = " \
	{getfield /params[{i}] value}
    end
    echo "which had a fitness of " {max_fitness}
end

// this is the function that actually runs the search
function pb_search
    int   i, v, p, chosen, mate
    float fit_chosen, fit_mate

    echo
    echo "   Fitness of individuals as they are evaluated:"
    echo "       (* indicates that this individual was added to the population)"
    echo

    for (i = 0; i < individuals && max_fitness < stopping_criterion; i = i + 1)
	if (actual_population < population)
	    // pick a random bit string
	    for (p = 0; p < parameters; p = p + 1)
		v = {rand 0 65535}
		setfield /params[{p}] bits {v}
	    end
	else
	    // pick random individual from population
	    chosen = {rand 0 {population}}
	    echo choosing {chosen}

	    // decide whether to do crossover
	    if ({rand 0.0 1.0} < crossover_prob)
		// choose a mate
		mate = {rand 0 {population}}
		while (mate == chosen)
		    mate = {rand 0 {population}}
		end
		echo mating with {mate}
		for (p = 0; p < parameters; p = p + 1)
		    fit_chosen = {getfield /population[{chosen}] fitness}
		    fit_mate = {getfield /population[{mate}] fitness}
		    if ({rand 0.0 1.0} < fit_chosen / (fit_chosen + fit_mate))
			setfield /params[{p}] bits \
			    {getfield /population[{chosen}]/gene[{p}] bits}
		    else
			setfield /params[{p}] bits \
			    {getfield /population[{mate}]/gene[{p}] bits}
		    end
		end
	    else
		for (p = 0; p < parameters; p = p + 1)
		    setfield /params[{p}] bits \
			{getfield /population[{chosen}]/gene[{p}] bits}
		end		
	    end

	    // mutate an existing bit string
	    for (p = 0; p < parameters; p = p + 1)
		echo par {p} originally {getfield /population[{chosen}]/gene[{p}] bits}
		setfield /params[{p}] bits \
		    {mutate {getfield /params[{p}] bits}}
		echo now {getfield /params[{p}] bits}
	    end
	end
	// assign this task to a worker
	delegate_task {i}
    end 
    finish

    print_results
end
